/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef ACTION_PL_H
#define ACTION_PL_H

#include "util/string.h"
#include "util/screen.h"
#include "util/c++11.h"

class Metrum3;
class View;
class ActionPoint;
class Object;

class ActionLine
{
public:
    virtual String name() const = 0;
    virtual Metrum3 base() const = 0;
    virtual Metrum3 tangent() const = 0;

    ScreenDist distance(const ScreenPos &, View *view) const;
    Metrum3 nearest(const ScreenPos &, View *view) const;
    bool doesIntersect(ActionLine *) const;
    Metrum3 intersection(ActionLine *) const;
};

class AngleActionLine final : public ActionLine
{
public:
    AngleActionLine(int angle, ActionPoint *point);

    String name() const;
    Metrum3 base() const;
    Metrum3 tangent() const;

private:
    ActionPoint *const the_point;
    int const the_angle;
};

class ObjectActionLine : public ActionLine
{
public:
    ObjectActionLine(const String &name, Object *object);

    String name() const;
    Metrum3 base() const;
    Metrum3 tangent() const;

private:
    Object *const the_object;
    String const the_name;
};

class ActionPoint
{
public:
    ActionPoint(const String &name, Object *object);
    ~ActionPoint();

    Object *object() const;
    String name() const;
    Metrum3 pos() const;

    List<ActionLine *> lines();
    void move(const Metrum3 &pos);

private:
    Object *const the_object;
    String const the_name;

    List<AngleActionLine *> angle_lines;
};

#endif
