/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef DRAWING_H
#define DRAWING_H

#include <toolkit/stdmodel.h>
#include "util/metrum.h"
#include "util/string.h"

class DrawingItem;
class ObjectContext;
class Object;
class ActionGroup;
class XMLWriter;
class Metrum3;
class Action;

class Drawing
{
public:
    enum FileType {
	GenericFile,
	SchematicFile, SymbolFile, LetterFile,
	NeverballFile
    };

private:
    FileType const file_type;
    DrawingItem *const item;
    String letter_character;
    Metrum letter_width;
    bool letter_space;
    String map_text;

    int64_t the_base;
    int base_base, base_exponent;

    ObjectContext *const root_context;
    List<Object *> the_objects, all_objects;
    void walk_context(ObjectContext *context);

    ActionGroup *const the_root, *most_recent, *last_saved;
    List<ActionGroup *> chain;

public:
    Drawing(FileType fileType, DrawingItem *item);
    ~Drawing();

    void write(XMLWriter *writer, Drawing *embedded_into = 0);

    FileType fileType() const;
    String letterCharacter() const;
    void setLetterCharacter(const String &letterCharacter);
    Metrum letterWidth() const;
    void setLetterWidth(Metrum letterWidth);
    bool letterSpace() const;
    void setLetterSpace(bool letterSpace);
    void setMapText(const String &mapText);

    int64_t base() const;
    int baseBase() const;
    void setBaseBase(int baseBase);
    int baseExponent() const;
    void setBaseExponent(int baseExponent);

    Metrum parseMetrum(const String &val, bool &ok) const;
    Metrum3 parseMetrum3(const String &val, bool &ok) const;
    Metrum scale(double eff) const;
    Metrum3 scale(double xeff, double yeff, double zeff) const;
    double eff(Metrum m) const;
    String format(Metrum m) const;
    String format(const Metrum3 &m) const;

    ObjectContext *rootContext() const;
    List<Object *> objects() const;
    void updateObjectList();
    void registerObject(Object *object);
    void unregisterObject(Object *object);

    ActionGroup *head() const;
    ActionGroup *root() const;
    void seek(ActionGroup *ag);
    void performActions(const List<Action *> &actions,
			Icon *icon, const String &name);
    bool isModified() const;

    ActionGroup *mostRecent() const;
    ActionGroup *lastSaved() const;
    void setLastSaved();

    StandardInterior historyInterior;
};

#endif
