/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <config.h>
#include <stdio.h>
#include <errno.h>
#include <err.h>
#include "filemonitor.h"
#include "util/string.h"
#include "util/c++11.h"
#include "../gettext.h"

#define _(str) gettext(str)


FileMonitorListener::FileMonitorListener() : end_exist(false)
{
}

void FileMonitorListener::event(const String &filename, FAMCodes code)
{
    if (end_exist)
	regularEvent(filename, code);
    else
	existsEvent(filename, code);

    if (code == FAMEndExist)
	end_exist = true;
}

void FileMonitorListener::existsEvent(
    const String &/*filename*/, FAMCodes /*code*/)
{
}

void FileMonitorListener::regularEvent(
    const String &/*filename*/, FAMCodes /*code*/)
{
}

/****************************************************************************/

#ifdef HAVE_LIBFAM

class FileMonitor::Internal final : public Toolkit::EventLoop::Reader
{
    FAMConnection *open_fam_connection() {
	FAMConnection *fc = (FAMConnection *)calloc(1, sizeof(FAMConnection));

	if (FAMOpen2(fc, program_invocation_name)) {
	    free(fc);
	    throw Error();
	}

	return fc;
    }
public:
    Internal() : Toolkit::EventLoop::Reader(
	FAMCONNECTION_GETFD(fc = open_fam_connection())) {
    }
    ~Internal() {
	if (FAMClose(fc))
	    warnx("FAMClose failed");

	free(fc);
    }
    void read() {
	int ret;

	while ((ret = FAMPending(fc)) == 1) {
	    FAMEvent fe;

	    if (FAMNextEvent(fc, &fe) != 1)
		warnx("FAMNextEvent failed");
	    else
		static_cast<FileMonitorListener *>(fe.userdata)
		    ->event(fe.filename, fe.code);
	}

	if (ret == -1)
	    warnx("FAMPending failed");
    }

    FAMConnection *fc;
    std::map<FileMonitorListener *, FAMRequest *> fr;

    class Error : public std::exception {
    };
};

FileMonitor::FileMonitor() : i(0)
{
    try {
	i = new Internal();
    } catch (Internal::Error &e) {
        fprintf(stderr, _("Cound not connect to FAM daemon. "
			  "Running without FAM support...\n"));
    }
}

FileMonitor::~FileMonitor()
{
    delete i;
}

void FileMonitor::addListener(FileMonitorListener *listener,
			      const String &filename)
{
    if (!i)
	return;

    i->fr[listener] = (FAMRequest *)calloc(1, sizeof(FAMRequest));

    if (FAMMonitorDirectory(i->fc, filename.data(), i->fr[listener], listener))
	warnx("FAMMonitorDirectory failed");
}

void FileMonitor::removeListener(FileMonitorListener *listener)
{
    if (!i)
	return;

    if (FAMCancelMonitor(i->fc, i->fr[listener]))
	warnx("FAMCancelMonitor failed");

    free(i->fr[listener]);
    i->fr.erase(listener);
}

#else

FileMonitor::FileMonitor() : i(0) {}
FileMonitor::~FileMonitor() {}
void FileMonitor::addListener(FileMonitorListener *, const String &) {}
void FileMonitor::removeListener(FileMonitorListener *) {}

#endif /* HAVE_LIBFAM */
