/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef GUI_ACTIONSETS_H
#define GUI_ACTIONSETS_H

#include <toolkit/toolkit.h>
#include "util/list.h"

class Hub;
class String;
class Item3;

class FileActionSet : public Toolkit::ActionSet
{
    Hub &hub;
    Toolkit::Connection &conn;
    Toolkit::Action new_schematic_action;
    Toolkit::Action new_neverball_action;
    Toolkit::MenuBuilder file_types_builder;
    List<String> old_recent_files;
    List<Toolkit::Action *> old_recent_file_actions;
    List<Item3 *> new_recent_files;
    List<Toolkit::Action *> new_recent_file_actions;
    Toolkit::Action no_recent_files_action;
    Toolkit::MenuBuilder recent_files_builder;
    void visit_path(const String &path);
public:
    FileActionSet(Hub &hub, Toolkit::Connection &conn);
    ~FileActionSet();
    void visitSomeFile();
    void actionTriggered(Toolkit::Action *action);

    void handleChanged();
    void drawingContentsChanged();

    Toolkit::Action newFileAction;
    Toolkit::Action newFolderAction;
    Toolkit::Action recentFilesAction;
    Toolkit::Action saveAction;
    Toolkit::Action saveAllAction;
    Toolkit::Action commitAction;
    // Toolkit::Action gitAction;
    // Toolkit::Action renameAction;
    // Toolkit::Action deleteAction;
    Toolkit::Action printAction;
    Toolkit::Action sendAction;
    Toolkit::Action visitComponentAction;
};

class DrawingActionSet : public Toolkit::ActionSet
{
    Hub &hub;
public:
    DrawingActionSet(Hub &hub);
    void actionTriggered(Toolkit::Action *action);
    void updateActions();

    Toolkit::Action undoAction;
    Toolkit::Action redoAction;
    Toolkit::Action zoomInAction;
    Toolkit::Action zoomOutAction;
};

class PropertiesActionSet : public Toolkit::ActionSet
{
    Hub &hub;
    Toolkit::PropertiesView &view;
public:
    PropertiesActionSet(Hub &hub, Toolkit::PropertiesView &view);
    void actionTriggered(Toolkit::Action *action);
    void updateActions();

    Toolkit::Action selectAllAction;
    Toolkit::Action resetAction;
};

class PopupActionSet : public Toolkit::ActionSet
{
    Hub &hub;
public:
    PopupActionSet(Hub &hub);
    void actionTriggered(Toolkit::Action *action);

    Toolkit::Action deselectModeAction;
};

#endif
