/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef GUI_VIEWPORT_H
#define GUI_VIEWPORT_H

#include <toolkit/toolkit.h>
#include "util/screen.h"

class Hub;
class ViewportScroller; /* internal */
class ActionGroup;
class View;

class ViewportNavigator : public Toolkit::Navigator
{
    Hub &hub;
    double const scale;
    Toolkit::NavigatorData getNavigatorData() const;
    void navigateTo(int x, int y);
public:
    ViewportNavigator(Toolkit::Connection &conn, Hub &hub, double scale);
};

class Viewport : public Toolkit::Widget, Toolkit::Focussable
{
    ViewportScroller *horz_scroller, *vert_scroller;
    Hub &hub;
    bool show_origin, show_grid, crosshairs_cursor;
    unsigned int buttons_down;
    X::Cursor no_cursor, move_cursor;
    bool has_focus;
    bool pointer_inside, pointer_grabbed;
    ScreenPos entry_pos, last_pos;
    bool significantly_moved;
    bool its_us;
    X::Pixmap buffer;
    int buffer_width, buffer_height;
    ActionGroup *buffer_state;
    View *buffer_view;
    ScreenPos screen_pos(int x, int y) const;
public:
    Viewport(Toolkit::Connection &conn, Hub &hub);
    ~Viewport();
    Toolkit::Menu *contextMenu;

    bool pointerActive() const;
    ScreenPos pointer() const;

    bool showOrigin() const;
    void setShowOrigin(bool showOrigin);
    bool showGrid() const;
    void setShowGrid(bool showGrid);
    bool crosshairsCursor() const;
    void setCrosshairsCursor(bool crosshairsCursor);

    void updateMinorMode();
    void updateScrollPos();

    void updateLayout();
    void paint(const Toolkit::Region &region, X::Pixmap pixmap);
    void enterEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state);
    void leaveEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state);
    void motionEvent(Time time, int x, int y, int x_root, int y_root,
		     unsigned int state, char is_hint);
    void buttonPressEvent(Time time, int x, int y,
			  int x_root, int y_root,
			  unsigned int state, unsigned int button);
    void buttonReleaseEvent(Time time, int x, int y,
			    int x_root, int y_root,
			    unsigned int state, unsigned int button);
    void keyPressEvent(Time time, int x, int y,
		       int x_root, int y_root,
		       unsigned int state, unsigned int keycode,
		       const std::string &text, KeySym keysym);
    void keyReleaseEvent(Time time, int x, int y,
			 int x_root, int y_root,
			 unsigned int state, unsigned int keycode);
    void focusInEvent();
    void focusOutEvent();
};

#endif
