/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef HUB_H
#define HUB_H

#include <map>
#include "util/string.h"

class Item3;
class MajorMode;
class ActionGroup;
class MinorMode;
class Object;
class Drawing;
class View;
class ToolboxModel; /* internal */
class AttributeModel; /* internal */
class ItemInterior;
class ScreenPos;

class Hub
{
    Item3 *_handle;
    MajorMode *major_mode;
    ActionGroup *drawing_head;
    bool drawing_is_modified;
    MinorMode *minor_mode;
    List<Object *> selected_objects;
    String status_message;
    mutable std::map<Drawing *, View *> views;

    ToolboxModel *const toolbox_model;
    AttributeModel *const attribute_model;

    virtual void updateHandle() = 0;
    virtual void updateDrawingStatus() = 0;
    virtual void updateView() = 0;
    virtual void updateMajorMode() = 0;
    virtual void updateMinorMode() = 0;
    virtual void updateSelectedObjects() = 0;
    virtual void updateStatusMessage() = 0;

public:
    Hub();
    ~Hub();
    Item3 *handle() const;
    Drawing *drawing() const;
    View *view() const;
    MajorMode *majorMode() const;
    ItemInterior *toolbox() const;
    ItemInterior *attributes() const;

    virtual bool pointerActive() const = 0;
    virtual ScreenPos pointer() const = 0;

    /* called by Shared */
    void drawingStatusChanged();
    virtual void updateProjectDirectories() = 0;
    virtual void requestClose() = 0;

    /* called by user interface */
    void setHandle(Item3 *handle);
    void setView(View *view);
    void setMajorMode(MajorMode *majorMode);
    void deselectMajorMode();
    MajorMode *componentMajorMode() const;
    void selectObjects(const List<Object *> &objects);
    virtual String selectedLibraryComponent() const = 0;

    /* called by MajorMode */
    void minorModeChanged();
    void selectedObjectsChanged();
    virtual void update() = 0;

    /* called by MinorMode */
    void statusMessageChanged();
};

#endif
