/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <toolkit/stdmodel.h>
#include "nb_attribute.h"
#include "icons.h"
#include "variant.h"


static StandardItem coin_value_item_1(_("1 (yellow)"), icons::tree_leaf);
static StandardItem coin_value_item_5(_("5 (red)"), icons::tree_leaf);
static StandardItem coin_value_item_10(_("10 (blue)"), icons::tree_leaf);

namespace NeverballAttributes {
    Attribute symbol(_("Symbol")),
	      travelTime(_("Travel time")),
	      path(_("Path")),
	      enableFlag(_("Enable flag")),
	      model(_("Model")),
	      value(_("Value")),
	      radius(_("Radius")),
	      timer(_("Timer")),
	      state(_("State")),
	      detail(_("Detail"));
    StandardInterior coinValueEnum(List<Item *>() << &coin_value_item_1
						  << &coin_value_item_5
						  << &coin_value_item_10);
}

NeverballAttributeItems::NeverballAttributeItems(Hub &hub)
    : symbol(hub, NeverballAttributes::symbol),
      travelTime(hub, NeverballAttributes::travelTime),
      path(hub, NeverballAttributes::path),
      enableFlag(hub, NeverballAttributes::enableFlag),
      model(hub, NeverballAttributes::model),
      value(hub, NeverballAttributes::value),
      radius(hub, NeverballAttributes::radius),
      timer(hub, NeverballAttributes::timer),
      state(hub, NeverballAttributes::state),
      detail(hub, NeverballAttributes::detail)
{
}

CoinValueAttributeItem::CoinValueAttributeItem(Hub &hub, Attribute &attribute)
    : SelectorAttributeItem(hub, attribute, NeverballAttributes::coinValueEnum)
{
}

Item *CoinValueAttributeItem::selectedItem()
{
    switch (value().toInt()) {
    case 1: return choices.cachedItem(0);
    case 5: return choices.cachedItem(1);
    case 10: return choices.cachedItem(2);
    }
    return 0;
}

void CoinValueAttributeItem::setSelectedItem(Item *value)
{
    switch (choices.cachedIndexOf(value)) {
    case 0: setValue(Variant::fromInt(1)); break;
    case 1: setValue(Variant::fromInt(5)); break;
    case 2: setValue(Variant::fromInt(10)); break;
    }
}
