/*   
 * Copyright (C) 2003 Robert Kooima
 *
 * NEVERBALL is  free software; you can redistribute  it and/or modify
 * it under the  terms of the GNU General  Public License as published
 * by the Free  Software Foundation; either version 2  of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * General Public License for more details.
 */

#include "nb_matrix.h"
#include "nb_vector.h"

#define A 10
#define B 11
#define C 12
#define D 13
#define E 14
#define F 15

#define TINY 1e-5

/*---------------------------------------------------------------------------*/

void m_xps(float *M, const float *N)
{
    M[0] = N[0]; M[1] = N[4]; M[2] = N[8]; M[3] = N[C];
    M[4] = N[1]; M[5] = N[5]; M[6] = N[9]; M[7] = N[D];
    M[8] = N[2]; M[9] = N[6]; M[A] = N[A]; M[B] = N[E];
    M[C] = N[3]; M[D] = N[7]; M[E] = N[B]; M[F] = N[F];
}

int  m_inv(float *I, const float *M)
{
    float T[16];
    float d;

    T[0] = +(M[5] * (M[A] * M[F] - M[B] * M[E]) -
	     M[9] * (M[6] * M[F] - M[7] * M[E]) +
	     M[D] * (M[6] * M[B] - M[7] * M[A]));
    T[1] = -(M[4] * (M[A] * M[F] - M[B] * M[E]) -
	     M[8] * (M[6] * M[F] - M[7] * M[E]) +
	     M[C] * (M[6] * M[B] - M[7] * M[A]));
    T[2] = +(M[4] * (M[9] * M[F] - M[B] * M[D]) -
	     M[8] * (M[5] * M[F] - M[7] * M[D]) +
	     M[C] * (M[5] * M[B] - M[7] * M[9]));
    T[3] = -(M[4] * (M[9] * M[E] - M[A] * M[D]) -
	     M[8] * (M[5] * M[E] - M[6] * M[D]) +
	     M[C] * (M[5] * M[A] - M[6] * M[9]));

    T[4] = -(M[1] * (M[A] * M[F] - M[B] * M[E]) -
	     M[9] * (M[2] * M[F] - M[3] * M[E]) +
	     M[D] * (M[2] * M[B] - M[3] * M[A]));
    T[5] = +(M[0] * (M[A] * M[F] - M[B] * M[E]) -
	     M[8] * (M[2] * M[F] - M[3] * M[E]) +
	     M[C] * (M[2] * M[B] - M[3] * M[A]));
    T[6] = -(M[0] * (M[9] * M[F] - M[B] * M[D]) -
	     M[8] * (M[1] * M[F] - M[3] * M[D]) +
	     M[C] * (M[1] * M[B] - M[3] * M[9]));
    T[7] = +(M[0] * (M[9] * M[E] - M[A] * M[D]) -
	     M[8] * (M[1] * M[E] - M[2] * M[D]) +
	     M[C] * (M[1] * M[A] - M[2] * M[9]));

    T[8] = +(M[1] * (M[6] * M[F] - M[7] * M[E]) -
	     M[5] * (M[2] * M[F] - M[3] * M[E]) +
	     M[D] * (M[2] * M[7] - M[3] * M[6]));
    T[9] = -(M[0] * (M[6] * M[F] - M[7] * M[E]) -
	     M[4] * (M[2] * M[F] - M[3] * M[E]) +
	     M[C] * (M[2] * M[7] - M[3] * M[6]));
    T[A] = +(M[0] * (M[5] * M[F] - M[7] * M[D]) -
	     M[4] * (M[1] * M[F] - M[3] * M[D]) +
	     M[C] * (M[1] * M[7] - M[3] * M[5]));
    T[B] = -(M[0] * (M[5] * M[E] - M[6] * M[D]) -
	     M[4] * (M[1] * M[E] - M[2] * M[D]) +
	     M[C] * (M[1] * M[6] - M[2] * M[5]));

    T[C] = -(M[1] * (M[6] * M[B] - M[7] * M[A]) -
	     M[5] * (M[2] * M[B] - M[3] * M[A]) +
	     M[9] * (M[2] * M[7] - M[3] * M[6]));
    T[D] = +(M[0] * (M[6] * M[B] - M[7] * M[A]) -
	     M[4] * (M[2] * M[B] - M[3] * M[A]) +
	     M[8] * (M[2] * M[7] - M[3] * M[6]));
    T[E] = -(M[0] * (M[5] * M[B] - M[7] * M[9]) -
	     M[4] * (M[1] * M[B] - M[3] * M[9]) +
	     M[8] * (M[1] * M[7] - M[3] * M[5]));
    T[F] = +(M[0] * (M[5] * M[A] - M[6] * M[9]) -
	     M[4] * (M[1] * M[A] - M[2] * M[9]) +
	     M[8] * (M[1] * M[6] - M[2] * M[5]));

    d = M[0] * T[0] + M[4] * T[4] + M[8] * T[8] + M[C] * T[C];

    if (fabs(d) > TINY)
    {
	I[0] = T[0] / d;
	I[1] = T[4] / d;
	I[2] = T[8] / d;
	I[3] = T[C] / d;
	I[4] = T[1] / d;
	I[5] = T[5] / d;
	I[6] = T[9] / d;
	I[7] = T[D] / d;
	I[8] = T[2] / d;
	I[9] = T[6] / d;
	I[A] = T[A] / d;
	I[B] = T[E] / d;
	I[C] = T[3] / d;
	I[D] = T[7] / d;
	I[E] = T[B] / d;
	I[F] = T[F] / d;

	return 1;
    }
    return 0;
}

/*---------------------------------------------------------------------------*/

void m_basis(float *M,
	     const Vector3 &e0,
	     const Vector3 &e1,
	     const Vector3 &e2)
{
    M[0] = e0.x(); M[4] = e1.x(); M[8] = e2.x(); M[C] = 0.f;
    M[1] = e0.y(); M[5] = e1.y(); M[9] = e2.y(); M[D] = 0.f;
    M[2] = e0.z(); M[6] = e1.z(); M[A] = e2.z(); M[E] = 0.f;
    M[3] =    0.f; M[7] =    0.f; M[B] =    0.f; M[F] = 1.f;
}

/*---------------------------------------------------------------------------*/

void m_rot(float *M, const Vector3 &v, float a)
{
    Vector3 u;
    float U[16];
    float S[16];

    const float s = ((float) sin((double) a));
    const float c = ((float) cos((double) a));

    u = v_nrm(v);

    U[0] = u.x() * u.x(); U[4] = u.x() * u.y(); U[8] = u.x() * u.z();
    U[1] = u.y() * u.x(); U[5] = u.y() * u.y(); U[9] = u.y() * u.z();
    U[2] = u.z() * u.x(); U[6] = u.z() * u.y(); U[A] = u.z() * u.z();

    S[0] =    0.f; S[4] = -u.z(); S[8] =  u.y();
    S[1] =  u.z(); S[5]  =   0.f; S[9] = -u.x();
    S[2] = -u.y(); S[6] =  u.x(); S[A] =    0.f;

    M[0] = U[0] + c * (1.f - U[0]) + s * S[0];
    M[1] = U[1] + c * (0.f - U[1]) + s * S[1];
    M[2] = U[2] + c * (0.f - U[2]) + s * S[2];
    M[3] = 0.f;
    M[4] = U[4] + c * (0.f - U[4]) + s * S[4];
    M[5] = U[5] + c * (1.f - U[5]) + s * S[5];
    M[6] = U[6] + c * (0.f - U[6]) + s * S[6];
    M[7] = 0.f;
    M[8] = U[8] + c * (0.f - U[8]) + s * S[8];
    M[9] = U[9] + c * (0.f - U[9]) + s * S[9];
    M[A] = U[A] + c * (1.f - U[A]) + s * S[A];
    M[B] = 0.f;
    M[C] = 0.f;
    M[D] = 0.f;
    M[E] = 0.f;
    M[F] = 1.f;
}

/*---------------------------------------------------------------------------*/

Vector3 m_vxfm(const float *M, const Vector3 &w)
{
    return Vector3(w.x() * M[0] + w.y() * M[4] + w.z() * M[8],
		   w.x() * M[1] + w.y() * M[5] + w.z() * M[9],
		   w.x() * M[2] + w.y() * M[6] + w.z() * M[A]);
}

