/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef SCH_ATTRIBUTE_H
#define SCH_ATTRIBUTE_H

#include <toolkit/stdmodel.h>
#include "util/metrum.h"
#include "attribute.h"

class Drawing;
class XMLWriter;

namespace SchematicEnum {
    extern List<String> boolean;
    extern List<String> capstyle;
    extern List<String> dashstyle;
    extern List<String> filltype;
    extern List<String> show_name_value;
    extern List<String> alignment;
    extern List<String> pintype;
    extern List<String> color;
}

class SchematicAttributes
{
public:
    SchematicAttributes();

    Attribute *const color, *const lineWidth, *const capStyle,
	      *const dashStyle, *const dashLength, *const dashSpace,
	      *const fillType, *const fillWidth, *const fillAngle0,
	      *const fillPitch0, *const fillAngle1, *const fillPitch1,
	      *const isBus, *const isPin, *const isInverted, *const textSize,
	      *const visibility, *const showNameValue, *const angle,
	      *const alignment, *const selectable, *const mirror;

    StandardInterior colorEnum, capStyleEnum, dashStyleEnum, fillTypeEnum,
		     showNameValueEnum, angleEnum, alignmentEnum;
};

extern SchematicAttributes *schematicAttributes;

class SchematicAttributeItems
{
public:
    SchematicAttributeItems(Hub &hub);

    EnumAttributeItem color;
    MetrumAttributeItem lineWidth;
    EnumAttributeItem capStyle;
    EnumAttributeItem dashStyle;
    MetrumAttributeItem dashLength;
    MetrumAttributeItem dashSpace;
    EnumAttributeItem fillType;
    MetrumAttributeItem fillWidth;
    IntegerAttributeItem fillAngle0;
    MetrumAttributeItem fillPitch0;
    IntegerAttributeItem fillAngle1;
    MetrumAttributeItem fillPitch1;
    BoolAttributeItem isBus;
    BoolAttributeItem isPin;
    BoolAttributeItem isInverted;
    IntegerAttributeItem textSize;
    BoolAttributeItem visibility;
    EnumAttributeItem showNameValue;
    EnumAttributeItem angle;
    EnumAttributeItem alignment;
    BoolAttributeItem selectable;
    BoolAttributeItem mirror;
};

class SchematicLine
{
public:
    Drawing *const drawing;
    Metrum width;
    int cap_style;
    int dash_style;
    Metrum dash_length;
    Metrum dash_space;

    SchematicLine(Drawing *drawing);
    List<Attribute *> attributes() const;
    Variant attribute(const Attribute *attribute) const;
    Variant attributeDefault(const Attribute *attribute) const;
    void setAttribute(const Variant &value, const Attribute *attribute);
    void write(XMLWriter *writer);
};

class SchematicFill
{
public:
    Drawing *const drawing;
    int type;
    Metrum width;
    int angle0;
    Metrum pitch0;
    int angle1;
    Metrum pitch1;

    SchematicFill(Drawing *drawing);
    List<Attribute *> attributes() const;
    Variant attribute(const Attribute *attribute) const;
    Variant attributeDefault(const Attribute *attribute) const;
    void setAttribute(const Variant &value, const Attribute *attribute);
    void write(XMLWriter *writer);
};

#endif
