/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef SCH_CIRCLE_H
#define SCH_CIRCLE_H

#include "majormode.h"
#include "object.h"
#include "sch_attribute.h"
#include "action_pl.h"

class CircleMajorMode : public MajorMode
{
public:
    CircleMajorMode(Hub &hub);
    void paint(Painter &painter);
    List<ActionLine *> lines(ActionPoint *handle);

    void accept(MinorMode *minor_mode);
    void cancel();

private:
    PointSelectMinorMode posMinorMode;
    PointSelectMinorMode radiusMinorMode;
};

class CircleObject : public Object
{
public:
    CircleObject(const Metrum3 &pos, Metrum radius, Drawing *drawing);
    String name() const;
    void paint(Painter &painter, const Embedding &embedding);
    void paintShadow(const Metrum3 &pos, const ActionPoint *handle,
		     Painter &painter);

    List<ActionPoint *> handles();
    Metrum3 handlePos(const ActionPoint *handle) const;
    void moveHandle(const Metrum3 &pos, const ActionPoint *handle);
    ScreenDist proximity(const ScreenPos &point, View *view,
			 const Embedding &embedding) const;
    bool within(const ScreenRect &rect, View *view, bool partially,
		const Embedding &embedding) const;
    void write(XMLWriter *writer);
    List<Attribute *> attributes() const;
    Variant attribute(const Attribute *attribute) const;
    Variant attributeDefault(const Attribute *attribute) const;
    void setAttribute(const Variant &value, const Attribute *attribute);

    Metrum3 pos;
    Metrum radius;
    bool is_arc;
    int startangle, sweepangle;
    int color;
    SchematicLine line;
    SchematicFill fill;

private:
    ActionPoint posHandle, perimeterHandle;
};

#endif
