/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "sch_line.h"
#include "hub.h"
#include "xml_writer.h"
#include "drawing.h"
#include "variant.h"


LineMajorMode::LineMajorMode(Hub &hub) : LinearMajorMode(hub)
{
    setCaption(_("Line"));
}

Object *LineMajorMode::buildObject(const Metrum3 &p0, const Metrum3 &p1)
{
    return new LineObject(p0, p1, hub.drawing());
}

/****************************************************************************/

LineObject::LineObject(const Metrum3 &p0, const Metrum3 &p1, Drawing *drawing)
    : LinearObject(p0, p1, drawing), color(3), line(drawing)
{
}

String LineObject::name() const
{
    return _("line");
}

void LineObject::write(XMLWriter *writer)
{
    writer->startElement("line");
    writer->writeAttribute("p0", drawing()->format(p0));
    writer->writeAttribute("p1", drawing()->format(p1));
    writer->writeAttribute("color", SchematicEnum::color[color]);
    line.write(writer);
    writer->endElement();
}

List<Attribute *> LineObject::attributes() const
{
    return List<Attribute *>() << schematicAttributes->color
			       << line.attributes();
}

Variant LineObject::attribute(const Attribute *attribute) const
{
    if (attribute == schematicAttributes->color)
	return Variant::fromInt(color);

    foreach (Attribute *attr, line.attributes())
	if (attr == attribute)
	    return line.attribute(attribute);

    return Variant();
}

Variant LineObject::attributeDefault(const Attribute *attribute) const
{
    if (attribute == schematicAttributes->color)
	return Variant::fromInt(3);

    foreach (Attribute *attr, line.attributes())
	if (attr == attribute)
	    return line.attributeDefault(attribute);

    return Variant();
}

void LineObject::setAttribute(const Variant &value,
			      const Attribute *attribute)
{
    if (attribute == schematicAttributes->color)
	color = value.toInt();

    foreach (Attribute *attr, line.attributes())
	if (attr == attribute)
	    line.setAttribute(value, attribute);
}
