/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "sch_path.h"
#include "util/metrum3.h"
#include "icons.h"
#include "xml_writer.h"
#include "variant.h"


Path1Object::Path1Object(Drawing *drawing)
    : Object(drawing), color(0), line(drawing), fill(drawing)
{
}

String Path1Object::name() const
{
    return _("path");
}

Icon *Path1Object::icon() const
{
    return icons::path;
}

void Path1Object::write(XMLWriter *writer)
{
    writer->startElement("path");
    writer->writeAttribute("d", join_strings(data, ' '));
    writer->writeAttribute("color", SchematicEnum::color[color]);
    line.write(writer);
    fill.write(writer);
    writer->endElement();
}

void Path1Object::paint(Painter &painter, const Embedding &embedding)
{
    /* not yet implemented */
}

void Path1Object::paintShadow(const Metrum3 &pos, const ActionPoint *handle,
			      Painter &painter)
{
    /* not yet implemented */
}

List<ActionPoint *> Path1Object::handles()
{
    return List<ActionPoint *>();
}

Metrum3 Path1Object::handlePos(const ActionPoint *handle) const
{
    return Metrum3();
}

List<ActionLine *> Path1Object::handleLines(const ActionPoint *handle)
{
    return List<ActionLine *>();
}

void Path1Object::moveHandle(const Metrum3 &pos, const ActionPoint *handle)
{
}

Metrum3 Path1Object::lineBase(const ActionLine *line) const
{
    return Metrum3();
}

Metrum3 Path1Object::lineTangent(const ActionLine *line) const
{
    return Metrum3(1, 0, 0);
}

ScreenDist Path1Object::proximity(const ScreenPos &point, View *view,
				  const Embedding &embedding) const
{
    return -1;
}

bool Path1Object::within(const ScreenRect &rect, View *view,
			 bool partially, const Embedding &embedding) const
{
    return false;
}

List<Attribute *> Path1Object::attributes() const
{
    return List<Attribute *>() << schematicAttributes->color
			       << line.attributes()
			       << fill.attributes();
}

Variant Path1Object::attribute(const Attribute *attribute) const
{
    if (attribute == schematicAttributes->color)
	return Variant::fromInt(color);

    foreach (Attribute *attr, line.attributes())
	if (attr == attribute)
	    return line.attribute(attribute);

    foreach (Attribute *attr, fill.attributes())
	if (attr == attribute)
	    return fill.attribute(attribute);

    return Variant();
}

Variant Path1Object::attributeDefault(const Attribute *attribute) const
{
    if (attribute == schematicAttributes->color)
	return Variant::fromInt(3);

    foreach (Attribute *attr, line.attributes())
	if (attr == attribute)
	    return line.attributeDefault(attribute);

    foreach (Attribute *attr, fill.attributes())
	if (attr == attribute)
	    return fill.attributeDefault(attribute);

    return Variant();
}

void Path1Object::setAttribute(const Variant &value,
			       const Attribute *attribute)
{
    if (attribute == schematicAttributes->color)
	color = value.toInt();

    foreach (Attribute *attr, line.attributes())
	if (attr == attribute)
	    line.setAttribute(value, attribute);

    foreach (Attribute *attr, fill.attributes())
	if (attr == attribute)
	    fill.setAttribute(value, attribute);
}
