/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "sch_picture.h"
#include "icons.h"
#include "hub.h"
#include "painter.h"
#include "xml_writer.h"
#include "drawing.h"
#include "sch_attribute.h"
#include "variant.h"


PictureMajorMode::PictureMajorMode(Hub &hub) : RectangularMajorMode(hub)
{
    setCaption(_("Picture"));
    setIcon(icons::picture);
}

Object *PictureMajorMode::buildObject(const Metrum3 &p0, const Metrum3 &p1)
{
    return new PictureObject(p0, p1, hub.drawing());
}

/****************************************************************************/

PictureObject::PictureObject(const Metrum3 &p0,
			     const Metrum3 &p1, Drawing *drawing)
    : RectangularObject(p0, p1, drawing),
      angle(0), mirrored(false), embedded(false)
{
}

String PictureObject::name() const
{
    return _("picture");
}

Icon *PictureObject::icon() const
{
    return icons::picture;
}

void PictureObject::paint(Painter &painter, const Embedding &embedding)
{
    RectangularObject::paint(painter, embedding);

    painter.paintLine(embedding.transform(p0),
		      embedding.transform(p1));
    painter.paintLine(Metrum3(embedding.transform(p0).x(),
			      embedding.transform(p1).y(),
			      embedding.transform(p0).z()),
		      Metrum3(embedding.transform(p1).x(),
			      embedding.transform(p0).y(),
			      embedding.transform(p1).z()));
}

void PictureObject::write(XMLWriter *writer)
{
    writer->startElement("picture");
    writer->writeAttribute("p0", drawing()->format(p0));
    writer->writeAttribute("p1", drawing()->format(p1));
    writer->writeAttribute("angle", int_to_str(angle));
    writer->writeAttribute("mirrored", SchematicEnum::boolean[mirrored]);
    writer->writeAttribute("embedded", SchematicEnum::boolean[embedded]);
    writer->writeAttribute("filename", filename);
    writer->writeAttribute("data", data);
    writer->endElement();
}

List<Attribute *> PictureObject::attributes() const
{
    return List<Attribute *>() << schematicAttributes->angle
			       << schematicAttributes->mirror;
}

Variant PictureObject::attribute(const Attribute *attribute) const
{
    if (attribute == schematicAttributes->angle)
	return Variant::fromInt(angle);

    if (attribute == schematicAttributes->mirror)
	return Variant::fromBool(mirrored);

    return Variant();
}

Variant PictureObject::attributeDefault(const Attribute *attribute) const
{
    if (attribute == schematicAttributes->angle)
	return Variant::fromInt(0);

    if (attribute == schematicAttributes->mirror)
	return Variant::fromBool(false);

    return Variant();
}

void PictureObject::setAttribute(const Variant &value,
				 const Attribute *attribute)
{
    if (attribute == schematicAttributes->angle)
	angle = value.toInt();

    if (attribute == schematicAttributes->mirror)
	mirrored = value.toBool();
}
