/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef SHARED_H
#define SHARED_H

#include <unistd.h>

class SharedPrivate; /* internal */
class Hub;
class Drawing;
class ItemInterior;
class String;
class Font1;
class FileItem;
class DirectoryItem;
class FileMonitor;
class Messages;

class Shared
{
    SharedPrivate *p;
public:
    Shared();
    ~Shared();
    void initialize();
    void requestQuit();

    void registerHub(Hub *hub);
    void unregisterHub(Hub *hub);
    void drawingStatusChanged(Drawing *drawing);

    ItemInterior *projectDirs() const;
    ItemInterior *library() const;

    bool moreInitialization(String &message);

    Drawing *component(const String &name) const;
    Font1 *font();

    FileItem *fileItem(const String &path);
    DirectoryItem *directoryItem(const String &path);

    FileMonitor *fileMonitor() const;

    ItemInterior &booleanEnum() const;

    class Sink
    {
    public:
	virtual ~Sink();
	virtual bool read(const void *buf, ssize_t count) = 0;
    };
    void readFile(const String &path, Sink *sink, Messages &messages);
};

extern Shared shared;

#endif
