/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef UTIL_METRUM3_H
#define UTIL_METRUM3_H

#include "metrum.h"

class Metrum3
{
public:
    Metrum3() : xp(0), yp(0), zp(0) {
    }
    Metrum3(Metrum xpos, Metrum ypos, Metrum zpos)
	: xp(xpos), yp(ypos), zp(zpos) {
    }

    bool isNull() const {
	return xp == 0 && yp == 0 && zp == 0;
    }

    Metrum x() const {
	return xp;
    }
    Metrum y() const {
	return yp;
    }
    Metrum z() const {
	return zp;
    }

    Metrum &rx() {
	return xp;
    }
    Metrum &ry() {
	return yp;
    }
    Metrum &rz() {
	return zp;
    }

    Metrum length() const;

    Metrum3 &operator+=(const Metrum3 &p) {
	xp += p.xp;
	yp += p.yp;
	zp += p.zp;
	return *this;
    }
    Metrum3 &operator-=(const Metrum3 &p) {
	xp -= p.xp;
	yp -= p.yp;
	zp -= p.zp;
	return *this;
    }
    Metrum3 &operator*=(double c) {
	xp *= c;
	yp *= c;
	zp *= c;
	return *this;
    }
    Metrum3 &operator/=(double c) {
	xp /= c;
	yp /= c;
	zp /= c;
	return *this;
    }

    bool operator==(const Metrum3 &p) const {
	return p.xp == xp && p.yp == yp && p.zp == zp;
    }
    bool operator!=(const Metrum3 &p) const {
	return p.xp != xp || p.yp != yp || p.zp != zp;
    }
    Metrum3 operator+(const Metrum3 &p) const {
	return Metrum3(xp + p.xp, yp + p.yp, zp + p.zp);
    }
    Metrum3 operator-(const Metrum3 &p) const {
	return Metrum3(xp - p.xp, yp - p.yp, zp - p.zp);
    }
    Metrum3 operator*(double c) const {
	return Metrum3(xp * c, yp * c, zp * c);
    }
    Metrum3 operator/(double c) const {
	return Metrum3(xp / c, yp / c, zp / c);
    }
    Metrum3 operator-() const {
	return Metrum3(-xp, -yp, -zp);
    }

private:
    Metrum xp, yp, zp;
};

inline Metrum3 operator*(double c, const Metrum3 &p)
{
    return p * c;
}

#endif
