/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <unistd.h>
#include <stdio.h>
#include "util/path.h"
#include "util/string.h"


bool path_contains(const String &path0, const String &path1)
{
    return path1 == path0 || path1.startsWith(path0 + "/");
}

bool path_is_sane(const String &path)
{
    return path.startsWith('/') && !path.endsWith('/') &&
	!path.contains("/./") && !path.endsWith("/.") &&
	!path.contains("/../") && !path.endsWith("/..");
}

String working_dir()
{
    char *wd = getcwd(NULL, 0);
    String str(wd);
    free(wd);
    return str;
}

String home_dir()
{
    char *home = getenv("HOME");
    if (!home || !*home)
	return "/";
    return home;
}

String sanitize_path(const String &path)
{
    List<String> tokens;
    unsigned int i = 0;

    if (path.startsWith('/'))
	tokens = path.split('/');
    else
	tokens = String(working_dir() + "/" + path).split('/');

    while (i < tokens.count()) {
	if (tokens[i] == "" || tokens[i] == ".")
	    tokens.removeAt(i);
	else if (tokens[i] == "..") {
	    if (i == 0) {
		fprintf(stderr, "Invalid path: '%s'\n", path.data());
		return "/";
	    }
	    tokens.removeRange(i - 1, 2);
	    i--;
	} else
	    i++;
    }

    return "/" + join_strings(tokens, '/');
}
