/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <err.h>
#include "util/regex.h"
#include "util/string.h"


Regex::Regex(const char *regex, int cflags)
{
    int errcode = regcomp(&preg, regex, cflags);

    if (errcode == 0)
	return;

    size_t errbuf_size = regerror(errcode, &preg, NULL, 0);
    char errbuf[errbuf_size];

    regerror(errcode, &preg, errbuf, errbuf_size);
    errx(EXIT_FAILURE, "regcomp: %s: %s", regex, errbuf);
}

Regex::~Regex()
{
    regfree(&preg);
}

List<String> Regex::exec(String const &str, size_t nmatch, int eflags)
{
    regmatch_t pmatch[nmatch];
    int errcode = regexec(&preg, str.data(), nmatch, pmatch, eflags);

    if (errcode == REG_NOMATCH)
	return List<String>();

    if (errcode != 0) {
	size_t errbuf_size = regerror(errcode, &preg, NULL, 0);
	char errbuf[errbuf_size];

	regerror(errcode, &preg, errbuf, errbuf_size);
	errx(EXIT_FAILURE, "regexec: %s: %s", str.data(), errbuf);
    }

    List<String> res;

    for (unsigned int i = 0; i < nmatch; i++)
	if (pmatch[i].rm_so != -1)
	    res.append(str.substr(pmatch[i].rm_so,
				  pmatch[i].rm_eo - pmatch[i].rm_so));
	else
	    res.append(String());

    return res;
}
