/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef UTIL_STRING_H
#define UTIL_STRING_H

#include <string>
#include "util/list.h"

class String;

String join_strings(const List<String> &list, char sep);
String join_strings(const List<String> &list, const String &sep);

int str_to_int(const String &str, bool &ok, int base = 10);
int64_t str_to_int64(const String &str, bool &ok, int base = 10);
float str_to_float(const String &str, bool &ok);
double str_to_double(const String &str, bool &ok);

String int_to_str(int val, int base = 10);
String int64_to_str(int64_t val, int base = 10);
String float_to_str(float val);
String double_to_str(double val);

String _(const char *msgid);

class String : public std::string
{
    void dump() const; /* for debugging purposes */

public:
    String() : std::string() {
    }
    String(const std::string &str) : std::string(str) {
    }
    String(const std::string &str, size_type pos, size_type n = npos)
	: std::string(str, pos, n) {
    }
    String(const char *s, size_type n) : std::string(s, n) {
    }
    String(const char *s) : std::string(s) {
    }
    String(size_type n, char c) : std::string(n, c) {
    }

    static String encode(const std::wstring &str);
    static String encodeUTF8(const std::wstring &str);
    std::wstring decode() const;
    std::wstring decodeUTF8() const;

    String arg(const String &a1) const;
    String arg(const String &a1, const String &a2) const;
    String capitalize() const;
    int count(char c) const;
    int count(const String &str) const;
    String &remove(char c);
    String &remove(const String &str);
    List<String> split(char sep) const;
    List<String> split(const String &sep) const;
    String trimmed() const;

    /* accessing */

    bool contains(char c) const {
	return find(c) != npos;
    }
    bool contains(const String &str) const {
	return find(str) != npos;
    }
    bool endsWith(char c) const {
	return !empty() && operator[](length() - 1) == c;
    }
    bool endsWith(const String &str) const {
	return length() >= str.length() &&
	    compare(length() - str.length(), str.length(), str) == 0;
    }
    size_type indexOf(char c, size_type from = 0) const {
	return find(c, from);
    }
    size_type indexOf(const String &str, size_type from = 0) const {
	return find(str, from);
    }
    bool isEmpty() const {
	return empty();
    }
    String left(int n) const {
	return substr(0, n);
    }
    String mid(size_type pos, size_type n = npos) const {
	return substr(pos, n);
    }
    String right(int n) const {
	return substr(length() - n);
    }
    bool startsWith(char c) const {
	return !empty() && operator[](0) == c;
    }
    bool startsWith(const String &str) const {
	return length() >= str.length() &&
	    compare(0, str.length() /* npos */, str) == 0;
    }

    /* modifying */

    String &prepend(char c) {
	insert(0, &c, 1);
	return *this;
    }
    String &prepend(const String &str) {
	insert(0, str);
	return *this;
    }
};

#endif
