/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef VARIANT_H
#define VARIANT_H

#include <assert.h>
#include "util/string.h"
#include "util/metrum.h"

class Variant
{
    enum ValueType { vNull, vString, vMetrum, vInt, vBool,
			    vFloat, vDouble, vPointer };
    ValueType type;
    String s;
    int64_t i;
    float f;
    double d;
    void *ptr;
    Variant(ValueType type, String const &s, int64_t i,
			    float f, double d, void *ptr);
public:
    Variant();
    bool operator==(Variant v);
    bool operator!=(Variant v);

    static Variant fromString(String const &s);
    static Variant fromMetrum(Metrum const &m);
    static Variant fromInt(int i);
    static Variant fromBool(bool b);
    static Variant fromFloat(float f);
    static Variant fromDouble(double d);
    template<typename T> static Variant fromPointer(T ptr) {
	return Variant(vPointer, String(), 0, 0., 0., ptr);
    }

    bool isNull() const;
    String toString() const;
    Metrum toMetrum() const;
    int toInt() const;
    bool toBool() const;
    float toFloat() const;
    double toDouble() const;
    template<typename T> T toPointer() const {
	assert(type == vPointer);
	return static_cast<T>(ptr);
    }
};

#endif
