/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef XML_H
#define XML_H

#include "util/metrum.h"
#include "util/string.h"
#include "shared.h"

#define NSSEP '$'

class XMLHandler;
class XMLElement;
class Metrum3;

class XMLSink : public Shared::Sink
{
    class Internal;
    Internal *const i;
public:
    XMLSink(XMLHandler *rootHandler, Messages &messages);
    ~XMLSink();
    bool read(const void *buf, ssize_t count);
};

class XMLHandler
{
public:
    virtual void element(XMLElement *element, Messages &messages);
    virtual void characterData(const String &data, Messages &messages);
    virtual void done(Messages &messages);
};

class XMLElement
{
public:
    XMLElement(const String &name,
	       const List<String> &att_names,
	       const List<String> &att_values,
	       Messages &messages);
    ~XMLElement();

    String namespaceURI() const;
    void setNamespaceURI(const String &uri);

    String name() const;
    bool selectAttribute(const String &name);
    String value() const;
    int toInt(int base = 10);
    int64_t toInt64(int base = 10);
    float toFloat();
    double toDouble();
    Metrum toMetrum(Drawing *drawing);
    Metrum3 toMetrum3(Drawing *drawing);
    int lookup(const List<String> &values);

    XMLHandler *handler() const;
    void setHandler(XMLHandler *handler);
    void ignore();

private:
    String uri;
    String const the_name;
    List<String> att_names;
    List<String> att_values;
    String att_value;
    XMLHandler *the_handler;
    Messages &messages;
};

#endif
