/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef XML_WRITER_H
#define XML_WRITER_H

#include <stdio.h>
#include "util/list.h"

class String;

/* Note: The caller must only pass characters that are valid in XML,
	 i.e. none of 00-1f (except 09, 0a, and 0d), d800-dfff, fffe,
	 ffff, or 110000 and above. */

class XMLWriter
{
    FILE *const fp;
    bool is_initialized;
    int has_root_element;
    List<String> stack;
    int pre_depth;
    enum {
	idle, character_data, start_tag
    } state;
    List<String> current_attrs;
    void write(const String &data);
    void escape(const String &data);
    void prepare_for_data();
public:
    XMLWriter(int fd);
    ~XMLWriter();
    void startElement(const String &name, bool preserve_whitespace = false);
    void endElement();
    void writeAttribute(const String &name, const String &value);
    void writeCharacterData(const String &data);
    void writeCDATASection(const String &data);
};

#endif
