/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "attmodel.h"
#include "icons.h"


EnumModel::EnumModel()
    : solid_item("solid", icons::tree_leaf),
      dotted_item("dotted", icons::tree_leaf),
      dashed_item("dashed", icons::tree_leaf),
      center_item("center", icons::tree_leaf),
      phantom_item("phantom", icons::tree_leaf), interior(items())
{
}

std::vector<Item *> EnumModel::items()
{
    std::vector<Item *> items;
    items.push_back(&solid_item);
    items.push_back(&dotted_item);
    items.push_back(&dashed_item);
    items.push_back(&center_item);
    items.push_back(&phantom_item);
    return items;
}

/****************************************************************************/

AttributeModel::AttributeModel()
    : geometry_item("Geometry", icons::tree_node),
      center_item("Center", icons::tree_leaf),
      x_item("X", icons::tree_leaf),
      y_item("Y", icons::tree_leaf),
      radius_item("Radius", icons::tree_leaf),
      attributes_item("Attributes", icons::tree_node),
      color_item("Color", icons::tree_leaf),

      line_item("Line style", icons::tree_node),
      line_width_item("Width", icons::tree_leaf),
      line_cap_style_item("Cap style", icons::tree_leaf),
      line_dash_style_item("Dash style", icons::tree_leaf),
      line_dash_length_item("Dash length", icons::tree_leaf),
      line_dash_spacing_item("Dash spacing", icons::tree_leaf),

      fill_item("Fill style", icons::tree_node),
      fill_type_item("Type", icons::tree_leaf),
      fill_width_item("Width", icons::tree_leaf),
      fill_angle0_item("Angle 1", icons::tree_leaf),
      fill_pitch0_item("Pitch/spacing 1", icons::tree_leaf),
      fill_angle1_item("Angle 2", icons::tree_leaf),
      fill_pitch1_item("Pitch/spacing 2", icons::tree_leaf),

      color_editor(&color_model.interior),
      line_cap_style_editor(&line_cap_style_model.interior),
      line_dash_style_editor(&line_dash_style_model.interior),
      fill_type_editor(&fill_type_model.interior),

      interior(items())
{
    center_item.setEditor(&center_editor);
    x_item.setEditor(&x_editor);
    y_item.setEditor(&y_editor);
    radius_item.setEditor(&radius_editor);
    color_item.setEditor(&color_editor);

    line_width_item.setEditor(&line_width_editor);
    line_cap_style_item.setEditor(&line_cap_style_editor);
    line_dash_style_item.setEditor(&line_dash_style_editor);
    line_dash_length_item.setEditor(&line_dash_length_editor);
    line_dash_spacing_item.setEditor(&line_dash_spacing_editor);

    fill_type_item.setEditor(&fill_type_editor);
    fill_width_item.setEditor(&fill_width_editor);
    fill_angle0_item.setEditor(&fill_angle0_editor);
    fill_pitch0_item.setEditor(&fill_pitch0_editor);
    fill_angle1_item.setEditor(&fill_angle1_editor);
    fill_pitch1_item.setEditor(&fill_pitch1_editor);

    center_item.interior.addItem(&x_item);
    center_item.interior.addItem(&y_item);

    geometry_item.interior.addItem(&center_item);
    geometry_item.interior.addItem(&radius_item);

    attributes_item.interior.addItem(&color_item);

    line_item.interior.addItem(&line_width_item);
    line_item.interior.addItem(&line_cap_style_item);
    line_item.interior.addItem(&line_dash_style_item);
    line_item.interior.addItem(&line_dash_length_item);
    line_item.interior.addItem(&line_dash_spacing_item);

    fill_item.interior.addItem(&fill_type_item);
    fill_item.interior.addItem(&fill_width_item);
    fill_item.interior.addItem(&fill_angle0_item);
    fill_item.interior.addItem(&fill_pitch0_item);
    fill_item.interior.addItem(&fill_angle1_item);
    fill_item.interior.addItem(&fill_pitch1_item);
}

std::vector<Item *> AttributeModel::items()
{
    std::vector<Item *> items;
    items.push_back(&geometry_item);
    items.push_back(&attributes_item);
    items.push_back(&line_item);
    items.push_back(&fill_item);
    return items;
}
