/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "editors.h"

#define EDITOR_PADDING 4
#define EDITOR_ICON_PADDING 4

using namespace Toolkit;


InputEditor::InputEditor() : input(0)
{
}

void InputEditor::paint(Connection &conn, X::GC gc,
			const Toolkit::Region &region,
			unsigned long background, X::Pixmap pixmap)
{
    gc.setForeground(conn.colors().viewItemText.pixel);
    pixmap.mbDrawString(conn.fontSet(), gc,
			region.x + EDITOR_PADDING,
			region.y + (region.height + conn.fontSet().ascent()
				      - conn.fontSet().descent()) / 2,
			value);
}

void InputEditor::createWidget(Connection &conn)
{
    input = new Input(conn);
    input->setText(value);
    input->selectNone();
    setWidget(input);
}

void InputEditor::deleteWidget()
{
    delete input;
    input = 0;
    setWidget(0);
}

void InputEditor::invoke(Time time, int x, int y, int x_root, int y_root)
{
    input->selectAll();
}

void InputEditor::accept()
{
    value = input->text();
    input->selectNone();
}

void InputEditor::reset()
{
    input->setText(value);
    input->selectNone();
}

/****************************************************************************/

SelectorEditor::SelectorEditor(ItemInterior *options)
    : options(options), selected_item(0), selector(0)
{
}

void SelectorEditor::paint(Connection &conn, X::GC gc,
			   const Toolkit::Region &region,
			   unsigned long background, X::Pixmap pixmap)
{
    if (!selected_item)
	return;

    Icon *icon = selected_item->icon();

    gc.setForeground(conn.colors().viewItemText.pixel);
    pixmap.wcDrawString(conn.fontSet(), gc,
			region.x + EDITOR_PADDING +
			    icon->width() + EDITOR_ICON_PADDING,
			region.y +
			    (region.height + conn.fontSet().ascent()
					   - conn.fontSet().descent()) / 2,
			selected_item->caption());

    X::Drawable::copyArea(icon->pixmap(conn, background),
			  pixmap, gc, 0, 0, icon->width(), icon->height(),
			  region.x + EDITOR_PADDING,
			  region.y + (region.height - icon->height()) / 2);
}

void SelectorEditor::createWidget(Connection &conn)
{
    selector = new Selector(options, conn);
    selector->setSelectedItem(selected_item);
    setWidget(selector);
}

void SelectorEditor::deleteWidget()
{
    delete selector;
    selector = 0;
    setWidget(0);
}

void SelectorEditor::invoke(Time time, int x, int y, int x_root, int y_root)
{
    selector->openMenu(time, x_root - x + selector->x(),
			     y_root - y + selector->y());
}

void SelectorEditor::accept()
{
    selector->closeMenu(CurrentTime, true);
    selected_item = selector->selectedItem();
}

void SelectorEditor::reset()
{
    selector->closeMenu(CurrentTime, false);
    selector->setSelectedItem(selected_item);
}
