/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <toolkit/connection.h>
#include "viewport.h"


ViewportNavigator::ViewportNavigator(Toolkit::Widget *widget)
    : Navigator(widget->conn), widget(widget)
{
}

Toolkit::NavigatorData ViewportNavigator::getNavigatorData() const
{
    X::Pixmap pixmap = window().createPixmap(
	100, 100, conn.visualInfo().depth);

    gc().setForeground(conn.screen().whitePixel());
    pixmap.fillRectangle(gc(), 0, 0, 100, 100);

    int origin_x = 50, origin_y = 50;

    gc().setForeground(conn.screen().blackPixel());
    pixmap.drawLine(gc(), origin_x - 10, origin_y - 10,
			  origin_x + 10, origin_y - 10);
    pixmap.drawLine(gc(), origin_x + 10, origin_y - 10,
			  origin_x + 10, origin_y + 10);
    pixmap.drawLine(gc(), origin_x + 10, origin_y + 10,
			  origin_x - 10, origin_y + 10);
    pixmap.drawLine(gc(), origin_x - 10, origin_y + 10,
			  origin_x - 10, origin_y - 10);

    return Toolkit::NavigatorData(widget->horzScroller()->position() / 10,
				  widget->vertScroller()->position() / 10,
				  widget->width() / 10,
				  widget->height() / 10, 100, 100, pixmap);
}

void ViewportNavigator::navigateTo(int x, int y)
{
    widget->horzScroller()->setPosition(x * 10);
    widget->vertScroller()->setPosition(y * 10);
}


class ViewportScroller : public Toolkit::WidgetScroller
{
    void updatePosition();
public:
    ViewportScroller(Toolkit::Widget &widget);
    int lineSize() const;
    void setRange(int range);
    void setPageSize(int pageSize);
};

ViewportScroller::ViewportScroller(Toolkit::Widget &widget)
    : WidgetScroller(widget)
{
}

int ViewportScroller::lineSize() const
{
    return 50;
}

void ViewportScroller::setRange(int range)
{
    WidgetScroller::setRange(range);
}

void ViewportScroller::setPageSize(int pageSize)
{
    WidgetScroller::setPageSize(pageSize);
}

void ViewportScroller::updatePosition()
{
    widget.invalidate();
}



Viewport::Viewport(Toolkit::Connection &conn)
    : Toolkit::Widget(conn, horz_scroller = new ViewportScroller(*this),
			    vert_scroller = new ViewportScroller(*this))
{
    horz_scroller->setRange(1000);
    vert_scroller->setRange(1000);
}

Viewport::~Viewport()
{
    delete horz_scroller;
    delete vert_scroller;
}

void Viewport::updateLayout()
{
    horz_scroller->setPageSize(width());
    horz_scroller->setPosition((1000 - width()) / 2);
    vert_scroller->setPageSize(height());
    vert_scroller->setPosition((1000 - height()) / 2);
}

void Viewport::paint(const Toolkit::Region &region, X::Pixmap pixmap)
{
    int x = this->x() - region.x;
    int y = this->y() - region.y;

    gc().setForeground(conn.screen().whitePixel());
    pixmap.fillRectangle(gc(), x, y, width(), height());

    int origin_x = x + 500 - horzScroller()->position();
    int origin_y = y + 500 - vertScroller()->position();

    gc().setForeground(conn.screen().blackPixel());
    pixmap.drawLine(gc(), origin_x - 100, origin_y - 100,
			  origin_x + 100, origin_y - 100);
    pixmap.drawLine(gc(), origin_x + 100, origin_y - 100,
			  origin_x + 100, origin_y + 100);
    pixmap.drawLine(gc(), origin_x + 100, origin_y + 100,
			  origin_x - 100, origin_y + 100);
    pixmap.drawLine(gc(), origin_x - 100, origin_y + 100,
			  origin_x - 100, origin_y - 100);
}
