/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "buttons.h"
#include "connection.h"

#define BUTTON_PADDING 12

using namespace Toolkit;


ButtonicWidget::CornerRoundness::CornerRoundness()
    : topLeft(0), topRight(0), bottomLeft(0), bottomRight(0)
{
}

ButtonicWidget::CornerRoundness::CornerRoundness(int r)
    : topLeft(r), topRight(r), bottomLeft(r), bottomRight(r)
{
}

ButtonicWidget::CornerRoundness::CornerRoundness(
    int topLeft, int topRight, int bottomLeft, int bottomRight)
    : topLeft(topLeft), topRight(topRight),
      bottomLeft(bottomLeft), bottomRight(bottomRight)
{
}

/****************************************************************************/

ButtonicWidget::ButtonicWidget(Connection &conn)
    : Widget(conn), decoration_width(0), decoration_height(0),
      highlighted(false)
{
}

void ButtonicWidget::paint(const Region &region, X::Pixmap pixmap)
{
    int x = this->x() - region.x;
    int y = this->y() - region.y;

    int text_x = x + (width() - decorationWidth()) / 2;
    int text_y = y + (height() - decorationHeight()) / 2;

    paintOuterBorder(colors().buttonBorder.pixel,
		     colors().buttonBorderPixel.pixel, region, pixmap);
    paintInnerBorder(colors().buttonRaised.pixel,
		     colors().buttonLowered.pixel, region, pixmap);

    if (highlighted)
	gc().setForeground(colors().buttonHighlight.pixel);
    else
	gc().setForeground(colors().background.pixel);

    pixmap.fillRectangle(gc(), x + 2, y + 2, width() - 4, height() - 4);

    paintDecoration(text_x, text_y, pixmap);
}

void ButtonicWidget::enterEvent(Time /*time*/, int /*x*/, int /*y*/,
				int /*x_root*/, int /*y_root*/,
				unsigned int /*state*//*, char*/ /*is_hint*/)
{
    highlighted = true;
    invalidate();
}

void ButtonicWidget::leaveEvent(Time /*time*/, int /*x*/, int /*y*/,
				int /*x_root*/, int /*y_root*/,
				unsigned int /*state*//*, char*/ /*is_hint*/)
{
    highlighted = false;
    invalidate();
}

void ButtonicWidget::updateSizeRequirements()
{
    /* do nothing */
}

int ButtonicWidget::decorationWidth() const
{
    return decoration_width;
}

int ButtonicWidget::decorationHeight() const
{
    return decoration_height;
}

void ButtonicWidget::setDecorationSize(int width, int height)
{
    if (width == decoration_width && height == decoration_height)
	return;

    decoration_width = width;
    decoration_height = height;
    updateSizeRequirements();
}

void ButtonicWidget::paintOuterBorder(unsigned long color,
				      unsigned long pixelColor,
				      const Region &region, X::Pixmap pixmap)
{
    int x = this->x() - region.x;
    int y = this->y() - region.y;

    int tl = cornerRoundness().topLeft;
    int tr = cornerRoundness().topRight;
    int bl = cornerRoundness().bottomLeft;
    int br = cornerRoundness().bottomRight;

    gc().setForeground(color);
    pixmap.drawLine(gc(), x + tl, y, x + width() - tr - 1, y);
    pixmap.drawLine(gc(), x, y + tl, x, y + height() - bl - 1);
    pixmap.drawLine(gc(), x + bl, y + height() - 1,
		    x + width() - br - 1, y + height() - 1);
    pixmap.drawLine(gc(), x + width() - 1, y + tr,
		    x + width() - 1, y + height() - br - 1);

    if (tl == 2)
	pixmap.drawPoint(gc(), x + 1, y + 1);
    if (tr == 2)
	pixmap.drawPoint(gc(), x + width() - 2, y + 1);
    if (bl == 2)
	pixmap.drawPoint(gc(), x + 1, y + height() - 2);
    if (br == 2)
	pixmap.drawPoint(gc(), x + width() - 2, y + height() - 2);

    gc().setForeground(pixelColor);
    if (tl == 1)
	pixmap.drawPoint(gc(), x, y);
    if (tl == 2) {
	pixmap.drawPoint(gc(), x, y + 1);
	pixmap.drawPoint(gc(), x + 1, y);
    }
    if (tr == 1)
	pixmap.drawPoint(gc(), x + width() - 1, y);
    if (tr == 2) {
	pixmap.drawPoint(gc(), x + width() - 1, y + 1);
	pixmap.drawPoint(gc(), x + width() - 2, y);
    }
    if (bl == 1)
	pixmap.drawPoint(gc(), x, y + height() - 1);
    if (bl == 2) {
	pixmap.drawPoint(gc(), x, y + height() - 2);
	pixmap.drawPoint(gc(), x + 1, y + height() - 1);
    }
    if (br == 1)
	pixmap.drawPoint(gc(), x + width() - 1, y + height() - 1);
    if (br == 2) {
	pixmap.drawPoint(gc(), x + width() - 1, y + height() - 2);
	pixmap.drawPoint(gc(), x + width() - 2, y + height() - 1);
    }
}

void ButtonicWidget::paintInnerBorder(unsigned long color0,
				      unsigned long color1,
				      const Region &region, X::Pixmap pixmap)
{
    int x = this->x() - region.x;
    int y = this->y() - region.y;

    int tl = cornerRoundness().topLeft;
    int tr = cornerRoundness().topRight;
    int bl = cornerRoundness().bottomLeft;
    int br = cornerRoundness().bottomRight;

    if (tl == 0) tl = 1;
    if (tr == 0) tr = 1;
    if (bl == 0) bl = 1;
    if (br == 0) br = 1;

    gc().setForeground(color0);
    pixmap.drawLine(gc(), x + tl, y + 1, x + width() - tr - 1, y + 1);
    pixmap.drawLine(gc(), x + 1, y + tl, x + 1, y + height() - bl - 1);
    gc().setForeground(color1);
    pixmap.drawLine(gc(), x + bl, y + height() - 2,
		    x + width() - br - 1, y + height() - 2);
    pixmap.drawLine(gc(), x + width() - 2, y + tr,
		    x + width() - 2, y + height() - br - 1);
}

/****************************************************************************/

GenericButton::GenericButton(Connection &conn)
    : ButtonicWidget(conn), pressed(false), _insensitive(false)
{
}

bool GenericButton::insensitive() const
{
    return _insensitive;
}

void GenericButton::setInsensitive(bool insensitive)
{
    if (insensitive == _insensitive)
	return;

    _insensitive = insensitive;
    invalidate();
}

void GenericButton::paint(const Region &region, X::Pixmap pixmap)
{
    if (!pressed) {
	ButtonicWidget::paint(region, pixmap);
	return;
    }

    int x = this->x() - region.x;
    int y = this->y() - region.y;

    int text_x = x + (width() - decorationWidth()) / 2;
    int text_y = y + (height() - decorationHeight()) / 2;

    paintOuterBorder(colors().buttonBorder.pixel,
		     colors().buttonBorderPixel.pixel, region, pixmap);
    paintInnerBorder(colors().buttonDown.pixel,
		     colors().buttonDown.pixel, region, pixmap);

    gc().setForeground(colors().buttonDown.pixel);
    pixmap.fillRectangle(gc(), x + 2, y + 2, width() - 4, height() - 4);
    paintDecoration(text_x + 1, text_y + 1, pixmap);
}

void GenericButton::buttonPressEvent(
    Time /*time*/, int /*x*/, int /*y*/, int /*x_root*/, int /*y_root*/,
    unsigned int /*state*/, unsigned int button)
{
    if (button != Button1)
	return;

    if (!_insensitive) {
	pressed = true;
	invalidate();
    }
}

void GenericButton::buttonReleaseEvent(
    Time time, int x, int y, int x_root, int y_root,
    unsigned int state, unsigned int button)
{
    if (!pressed || button != Button1)
	return;

    pressed = false;
    invalidate();

    if (!_insensitive && !(state & ~(1 << (button + 7)) & ButtonMask))
	press(time, x, y, x_root, y_root, state);
}

void GenericButton::enterEvent(Time time, int x, int y,
			       int x_root, int y_root,
			       unsigned int state)
{
    pressed = !_insensitive && !!(state & Button1Mask);
    ButtonicWidget::enterEvent(time, x, y, x_root, y_root, state);
}

void GenericButton::leaveEvent(Time time, int x, int y,
			       int x_root, int y_root,
			       unsigned int state)
{
    pressed = false;
    ButtonicWidget::leaveEvent(time, x, y, x_root, y_root, state);
}

void GenericButton::press(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state)
{
    /* relax */
}

/****************************************************************************/

Button::Button(Connection &conn) : GenericButton(conn)
{
    setDecorationSize(0, conn.fontSet().ascent() +
			 conn.fontSet().descent());
}

std::string Button::text() const
{
    return _text;
}

void Button::setText(const std::string &text)
{
    _text = text;
    setDecorationSize(conn.fontSet().mbTextEscapement(text),
		      conn.fontSet().ascent() +
		      conn.fontSet().descent());
}

void Button::paintDecoration(int xpos, int ypos, X::Pixmap pixmap)
{
    gc().setForeground(colors().buttonText.pixel);
    pixmap.mbDrawString(conn.fontSet(), gc(),
			xpos, ypos + conn.fontSet().ascent(), text());
}

ButtonicWidget::CornerRoundness Button::cornerRoundness() const
{
    return CornerRoundness(2);
}

void Button::updateSizeRequirements()
{
    SizeRequirements reqs;
    reqs.minWidth = decorationWidth() + BUTTON_PADDING;
    reqs.minHeight = decorationHeight() + BUTTON_PADDING;
    setSizeRequirements(reqs);
}
