/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_CONTAINER0_H
#define TOOLKIT_CONTAINER0_H

#include "widget.h"

namespace Toolkit {

class ContainerWidget : public Widget
{
public:
    ContainerWidget(Connection &conn);
    ~ContainerWidget();
    void recursivelyUpdateLayout();
    void setMnemonicVisibility(bool visible);
    void windowPositionChanged();
    void paint(const Region &region, X::Pixmap pixmap);
    virtual void sizeRequirementsChanged(WidgetControl *control);

    class Control : public WidgetControl
    {
	WidgetControlScroller *horz_scroller, *vert_scroller;
	bool is_enabled;
	void sizeRequirementsChanged();
	void propagateChildLayoutRequest();
    public:
	Control(ContainerWidget &container);
	~Control();
	ContainerWidget &container;

	WidgetControlScroller *horzScroller() const;
	void setHorzScroller(WidgetControlScroller *horzScroller);
	WidgetControlScroller *vertScroller() const;
	void setVertScroller(WidgetControlScroller *vertScroller);

	bool isEnabled() const;
	void setEnabled(bool enable);

	X::Window window() const;
	X::GC gc() const;
	WidgetWindowHook *hook() const;

	void focus(Focussable *focussable);
	void focussablesChanged();
	void setDimensions(const Region &dimensions);
    };
protected:
    std::vector<Control *> registeredControls() const;
    std::vector<Control *> visibleControls() const;
private:
    std::vector<Control *> registered_controls, visible_controls;
    bool is_updating_layout;
    std::vector<Control *>::const_iterator const *child_update_iterator;
    void add_control(Control *control);
    void remove_control(Control *control);
    void enable_control(Control *control);
    void disable_control(Control *control);
    std::vector<Widget *> children() const;
};

} /* namespace Toolkit */

#endif
