/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <assert.h>
#include "docks.h"
#include "util/encoding.h"
#include "docks/logic.h"

using namespace Toolkit::Docks;


Dock::Dock(Manager &manager, const std::string &title, Icon *icon,
	   Widget *widget, const std::string &id,
	   unsigned long textColor, unsigned long backgroundColor)
    : manager(manager), action(*this),
      title(assert_decode(title)), icon(icon), widget(widget), id(id),
      textColor(textColor), backgroundColor(backgroundColor)
{
    manager.logic->addDock(this);

    action.setText(title);
    //action.setHotkey(hotkey);
    action.setIcon(icon);
}

Dock::~Dock()
{
    manager.logic->removeDock(this);
}

void Dock::actionTriggered(Action *action)
{
    assert(action == &this->action);
    manager.setFocussedDock(this);
}

/****************************************************************************/

Manager::Manager(Connection &conn) : logic(new Logic(conn))
{
}

Manager::~Manager()
{
    delete logic;
}

Toolkit::Widget *Manager::widget() const
{
    return logic->widget();
}

Toolkit::WidgetControl *Manager::mainControl() const
{
    return logic->mainControl();
}

Toolkit::Focussable *Manager::focusHandler() const
{
    return &logic->focusHandler;
}

Dock *Manager::focussedDock() const
{
    return logic->focussedDock();
}

void Manager::setFocussedDock(Dock *dock)
{
    logic->setFocussedDock(dock);
}

std::string Manager::configuration() const
{
    return logic->configuration();
}

void Manager::setConfiguration(const std::string &configuration)
{
    logic->setConfiguration(configuration);
}
