/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <errno.h>
#include "configuration.h"

using namespace Toolkit::Docks;


Configuration::Configuration(const std::string &str) : str(str), c(str.data())
{
}

Configuration::~Configuration()
{
}

bool Configuration::listFirst()
{
    if (*c != '(')
	throw Error();
    c++;
    if (*c != ')')
	return true;
    c++;
    return false;
}

bool Configuration::listNext()
{
    switch (*c) {
    case ',':
	c++;
	if (*c != ' ')
	    throw Error();
	c++;
	return true;
    case ')':
	c++;
	return false;
    default:
	throw Error();
    }
}

void Configuration::consumeSpace()
{
    if (*c != ' ')
	throw Error();
    c++;
}

std::string Configuration::readNextToken()
{
    const char *p = c;
    c = strchr(c, ' ');
    if (c == NULL)
	throw Error();
    return std::string(p, c - p);
}

std::string Configuration::readLastToken()
{
    const char *p = c;
    const char *q0 = strchr(c, ')');
    const char *q1 = strchr(c, ',');
    c = q0 == NULL || (q1 != NULL && q1 < q0) ? q1 : q0;
    if (c == NULL)
	throw Error();
    return std::string(p, c - p);
}

bool Configuration::fullyConsumed() const
{
    return *c == '\0';
}

double Configuration::toDouble(const std::string &str)
{
    const char *nptr = str.data();
    char *endptr;
    double val;

    errno = 0;
    val = strtod(nptr, &endptr);
    if (errno != 0 || *nptr == '\0' || *endptr != '\0')
	throw Error();
    return val;
}

int Configuration::toInt(const std::string &str)
{
    const char *nptr = str.data();
    char *endptr;
    long int val;

    errno = 0;
    val = strtol(nptr, &endptr, 10);

    if (errno != 0 || *nptr == '\0' || *endptr != '\0' ||
	val < INT_MIN || val > INT_MAX)
	throw Error();

    return val;
}
