/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_DOCKS_FOCUS_H
#define TOOLKIT_DOCKS_FOCUS_H

#include "../docks.h"
#include "../container0.h"

class Toolkit::Docks::FocussableControl : public ContainerWidget::Control
{
    FocusHandler &focus_handler;
public:
    FocussableControl(ContainerWidget &container, FocusHandler &focusHandler);
    void focus(Focussable *focussable);
    void focussablesChanged();
};

class Toolkit::Docks::FocusHandler : public Focussable
{
    FocussableControl *focussed_control, *const default_control;
    std::vector<Focussable *> focussables;
    Focussable *focussed;
    bool has_focus;

    friend class FocussableControl; /* calls focus */
    void focus(FocussableControl *control, Focussable *focussable);
public:
    FocusHandler(FocussableControl &defaultControl);
    FocussableControl *focussedControl() const;
    void setFocussedControl(FocussableControl *control);
    void focussablesChanged();

    void keyPressEvent(Time time, int x, int y, int x_root, int y_root,
		       unsigned int state, unsigned int keycode,
		       const std::string &text, KeySym keysym);
    void keyReleaseEvent(Time time, int x, int y, int x_root, int y_root,
			 unsigned int state, unsigned int keycode);
    void focusInEvent();
    void focusOutEvent();
};

#endif
