/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "global.h"
#include "connection.h"
#include "widget.h"
#include "hooks.h"

using namespace Toolkit;


static X::Window create_window(
    Connection &conn, int x, int y, unsigned int width, unsigned int height,
    unsigned int border_width, unsigned long border_pixel, Window transient_for)
{
    X::Window window;

    XSetWindowAttributes xswa;

    xswa.border_pixel = border_pixel;
    xswa.override_redirect = true;
    xswa.save_under = true;

    window = conn.screen().rootWindow().createWindow(
	x, y, width, height, border_width,
	conn.visualInfo().depth, InputOutput, conn.visual(),
	CWBorderPixel | CWOverrideRedirect | CWSaveUnder, &xswa);

    if (transient_for)
	window.setTransientForHint(transient_for);

    window.selectInput(KeyPressMask | KeyReleaseMask |
		       ButtonPressMask | ButtonReleaseMask |
		       EnterWindowMask | LeaveWindowMask |
		       PointerMotionMask | ExposureMask |
		       VisibilityChangeMask | StructureNotifyMask |
		       FocusChangeMask);
    window.map();

    return window;
}

WidgetWindowHook *Toolkit::create_transient_window(
    Connection &conn, int x, int y, int width, int height,
    unsigned int border_width, unsigned long border_pixel,
    Widget *widget, WidgetWindowHook *transient_for, bool holds_menu)
{
    if (widget->sizeRequirements().minWidth > width)
	width = widget->sizeRequirements().minWidth;

    if (widget->sizeRequirements().minHeight > height)
	height = widget->sizeRequirements().minHeight;

    WidgetWindowHook *hook = new WidgetWindowHook(
	conn, create_window(conn, x, y, width, height, border_width,
			    border_pixel, transient_for->window()), NULL,
	x, y, width, height, border_width, transient_for, holds_menu);
    hook->control()->setWidget(widget);
    return hook;
}
