/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_TOOLBAR_H
#define TOOLKIT_TOOLBAR_H

#include <map>
#include "container1.h"
#include "menus.h"

namespace Toolkit {

class ToolbarButtonMenu; /* internal */

class ToolbarButton : public Widget, public Action::Observer
{
    bool highlighted, pressed;
    ToolbarButtonMenu *const menu;
public:
    ToolbarButton(Connection &conn, Action &action);
    ~ToolbarButton();
    void actionChanged();

    void paint(const Region &region, X::Pixmap pixmap);

    void enterEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state);
    void leaveEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state);
    void buttonPressEvent(Time time, int x, int y,
			  int x_root, int y_root,
			  unsigned int state, unsigned int button);
    void buttonReleaseEvent(Time time, int x, int y,
			    int x_root, int y_root,
			    unsigned int state, unsigned int button);
};

class ToolbarSeparator : public Widget
{
public:
    ToolbarSeparator(Connection &conn);
    void paint(const Region &region, X::Pixmap pixmap);
};

class Toolbar : public ContainerWidget
{
    VContainer menu_container;
    int full_width, expanding;
    std::map<Control *, bool> is_separator;
    std::map<Control *, Widget *> menu_widget;
    void update_size_requirements();
public:
    Toolbar(Connection &conn);
    ~Toolbar();
    Menu menu;
    void addWidgets(Widget *widget, Widget *menuWidget, bool isSeparator);

    void sizeRequirementsChanged(WidgetControl *control);
    void updateLayout();

    void paint(const Region &region, X::Pixmap pixmap);
    void buttonPressEvent(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state, unsigned int button);
};

class ToolbarBuilder
{
    std::vector<Widget *> toolbar_widgets;
    std::vector<Widget *> menu_widgets;
public:
    ToolbarBuilder(Connection &conn);
    ~ToolbarBuilder();
    Toolbar toolbar;
    void addAction(Action &action);
    void addSeparator();
    void addSpacer();
};

} /* namespace Toolkit */

#endif
