/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_VIEWS_HISTORYVIEW_H
#define TOOLKIT_VIEWS_HISTORYVIEW_H

#include "../model_view.h"
#include "../widget.h"

namespace Toolkit {

class HistoryView;

class HistoryViewShadow : public Shadow
{
    HistoryView &view;
    HistoryViewShadow *latest_child;
    unsigned int anchor_x, anchor_y;
    Region sensitive_region;
    unsigned int _indent, _height;
    unsigned int visible_children_count;
    unsigned int visible_children_height;

    Shadow *buildShadow(Item *item) const;
    void paint0(int origin_x, int origin_y, X::Pixmap pixmap);
    void paint1(int origin_x, int origin_y, X::Pixmap pixmap);

public:
    HistoryViewShadow(ItemInterior *interior, HistoryView &view);
    HistoryViewShadow(Item *item, Shadow *parent, HistoryView &view);

    HistoryViewShadow *latestChild() const;
    void setLatestChild(HistoryViewShadow *latestChild);

    void updateItem();
    void updateItems(unsigned int index, unsigned int old_count,
					 unsigned int new_count);

    HistoryViewShadow *parent() const;
    HistoryViewShadow *shadow(unsigned int index) const;
    HistoryViewShadow *find(int x, int y);
    HistoryViewShadow *nth(unsigned int index) const;

    unsigned int anchorX() const;
    unsigned int anchorY() const;

    unsigned int height() const;

    unsigned int visibleChildrenCount() const;
    unsigned int visibleChildrenHeight() const;

    void relayout(unsigned int x, unsigned int y);
    void paintChildren(int origin_x, int origin_y, X::Pixmap pixmap);
};

class HistoryView : public Widget, Focussable
{
    friend class HistoryViewShadow; /* calls isSelected */

    class Scroller;
    void scroll_position_changed();

    ItemInterior *root_interior;
    HistoryViewShadow *root_shadow, *latest_shadow;
    Scroller *vert_scroller;
    int origin_x, origin_y;

    virtual void setSelected(HistoryViewShadow *selected) = 0;
    virtual bool isSelected(HistoryViewShadow *shadow) const = 0;

public:
    HistoryView(Connection &conn);
    ~HistoryView();
    ItemInterior *rootInterior() const;
    void setRootInterior(ItemInterior *interior);
    HistoryViewShadow *rootShadow() const;
    HistoryViewShadow *latestShadow() const;
    void setLatestShadow(HistoryViewShadow *latest);

    void scrollTo(HistoryViewShadow *shadow);
    void updateLayout();

    void paint(const Region &region, X::Pixmap pixmap);
    void buttonPressEvent(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state, unsigned int button);
};

} /* namespace Toolkit */

#endif
