/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_VIEWS_PROPERTIESVIEW_H
#define TOOLKIT_VIEWS_PROPERTIESVIEW_H

#include "view.h"

namespace Toolkit {

class PropertiesView : public View, Focussable
{
    class Control;
    ViewShadow *_selected;
    ItemEditor *editor;
    Control *const editor_control;
    bool has_focus;
    bool editor_has_focus;
    int origin_x, origin_y;
    void show_editor_widget();
    void hide_editor_widget();
    ViewShadow *next_valid_shadow(ViewShadow *shadow);
    ViewShadow *previous_valid_shadow(ViewShadow *shadow);

    std::vector<Widget *> children() const;
protected:
    void focusThis();
public:
    PropertiesView(Connection &conn);
    ~PropertiesView();
    ViewShadow *selected() const;
    void setSelected(ViewShadow *shadow);

    void enterEditor(Time time, int x, int y, int x_root, int y_root);
    void leaveEditor(bool cancel);

    void recursivelyUpdateLayout();
    void updateLayout();
    void setMnemonicVisibility(bool visible);
    void windowPositionChanged();
    void paint(const Region &region, X::Pixmap pixmap);

    void buttonPressEvent(Time time, int x, int y,
			  int x_root, int y_root,
			  unsigned int state, unsigned int button);
    void keyPressEvent(Time time, int x, int y,
		       int x_root, int y_root,
		       unsigned int state, unsigned int keycode,
		       const std::string &text, KeySym keysym);
    void keyReleaseEvent(Time time, int x, int y,
			 int x_root, int y_root,
			 unsigned int state, unsigned int keycode);
    void focusInEvent();
    void focusOutEvent();

    ViewShadowLayout layoutShadow(ViewShadow *shadow);
    void paintShadow(ViewShadow *shadow,
		     int origin_x, int origin_y, X::Pixmap pixmap);
};

} /* namespace Toolkit */

#endif
