/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_VIEWS_VIEW_H
#define TOOLKIT_VIEWS_VIEW_H

#include "../model_view.h"
#include "../widget.h"

namespace Toolkit {

class View;

class ViewShadowLayout
{
public:
    ViewShadowLayout() : indent(0), height(0) {
    };
    Region sensitiveRegion, plupRegion, editorRegion;
    unsigned int indent, height;
};

class ViewShadow : public Shadow
{
    View &view;
    bool is_expanded;
    unsigned int anchor_x, anchor_y;
    ViewShadowLayout _layout;
    unsigned int visible_children_count;
    unsigned int visible_children_height;

    Shadow *buildShadow(Item *item) const;

public:
    ViewShadow(ItemInterior *interior, View &view);
    ViewShadow(Item *item, Shadow *parent, View &view);

    void updateItem();
    void updateItems(unsigned int index, unsigned int old_count,
					 unsigned int new_count);

    enum RegionType { SensitiveRegion, PlupRegion, EditorRegion };

    ViewShadow *parent() const;
    ViewShadow *shadow(unsigned int index) const;
    ViewShadow *last() const;
    ViewShadow *find(int x, int y, RegionType region);
    ViewShadow *nth(unsigned int index) const;
    ViewShadow *next(ViewShadow *shadow = 0);
    ViewShadow *previous(ViewShadow *shadow = 0);
    ViewShadow *lastDescendant();

    bool isExpanded() const;
    void setExpanded(bool expanded);

    unsigned int anchorX() const;
    unsigned int anchorY() const;

    const ViewShadowLayout &layout() const;

    unsigned int visibleChildrenCount() const;
    unsigned int visibleChildrenHeight() const;

    void relayout(unsigned int x, unsigned int y);
    void paint(int origin_x, int origin_y, X::Pixmap pixmap);
};

class View : public Widget
{
    class Scroller;
    void scroll_position_changed();

    ItemInterior *root_interior;
    ViewShadow *root_shadow;
    ViewShadow *highlighted_shadow;
    Scroller *vert_scroller;
    int origin_x, origin_y;
    int const padding_x, padding_y;

public:
    View(int padding_x, int padding_y, Connection &conn);
    ~View();

    ItemInterior *rootInterior() const;
    void setRootInterior(ItemInterior *interior);

    ViewShadow *rootShadow() const;
    ViewShadow *highlightedShadow() const;
    void scrollTo(ViewShadow *shadow);

    void updateLayout();

    void paint(const Region &region, X::Pixmap pixmap);
    void enterEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state/*, char is_hint*/);
    void leaveEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state/*, char is_hint*/);
    void motionEvent(Time time, int x, int y, int x_root, int y_root,
		     unsigned int state, char is_hint);
    void buttonPressEvent(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state, unsigned int button);

    virtual ViewShadowLayout layoutShadow(ViewShadow *shadow) = 0;
    virtual void paintShadow(ViewShadow *shadow,
			     int origin_x, int origin_y, X::Pixmap pixmap) = 0;
protected:
    ViewShadow *findShadow(int x, int y, ViewShadow::RegionType region);
    void toggleShadow(ViewShadow *shadow);
    int originX() const;
    int originY() const;
    virtual void highlightedShadowChanged();
};

} /* namespace Toolkit */

#endif
